@@ -65,8 +65,8 @@ class Controller():
             IS.store('scaff2gene2sequence', scaff2gene2sequence, 'pickle', 'Dicitonary of scaff -> gene -> nucleotide sequence')
 
         # Store the output
-        IS.generate('gene_info')
-        IS.generate("SNVs")
+        IS.generate('gene_info', **vargs)
+        IS.generate("SNVs", **vargs)
 
     def validate_input(self, args):
         '''
@@ -109,9 +109,10 @@ def profile_genes_wrapper(cmds):
         try:
             results.append(profile_genes(cmd.scaffold, **cmd.arguments))
         except Exception as e:
-            print(e)
-            traceback.print_exc()
             logging.error("FAILURE GeneException {0}".format(str(cmd.scaffold)))
+            logging.error(str(traceback.format_exc()))
+            logging.error(str(e))
+
             results.append(None)
     return results
 
@@ -583,10 +584,10 @@ def calc_gene_snp_counts(gdb, ldb, sdb, gene2sequence, scaffold=None):
     log_message += "\nSpecialPoint_genes {0} PID {1} SNP_counts_geneCalc end {2}".format(scaffold, pid, time.time())
 
     # Calculate dn/ds
-    GGdb['dNdS_substitutions'] = [((nC/nS) / (sC/sS)) if ((sC > 0) & (sS > 0)) else np.nan for nC, nS, sC, sS in zip(
+    GGdb.loc[:, 'dNdS_substitutions'] = [((nC/nS) / (sC/sS)) if ((sC > 0) & (sS > 0)) else np.nan for nC, nS, sC, sS in zip(
                                 GGdb['SNS_N_count'], GGdb['N_sites'],
                                 GGdb['SNS_S_count'], GGdb['S_sites'])]
-    GGdb['pNpS_variants'] = [((nC/nS) / (sC/sS)) if ((sC > 0) & (sS > 0)) else np.nan for nC, nS, sC, sS in zip(
+    GGdb.loc[:, 'pNpS_variants'] = [((nC/nS) / (sC/sS)) if ((sC > 0) & (sS > 0)) else np.nan for nC, nS, sC, sS in zip(
                                     GGdb['SNV_N_count'], GGdb['N_sites'],
                                     GGdb['SNV_S_count'], GGdb['S_sites'])]
     # GGdb['SNPs_per_bp'] = [x/y if y > 0 else np.nan for x, y in \
@@ -613,11 +614,11 @@ def Characterize_SNPs_wrapper(Ldb, gdb, gene2sequence):
     # Get a non-nonredundant list of SNPs
     Sdb = Ldb.drop_duplicates(subset=['scaffold', 'position'], keep='last')\
                 .sort_index().drop(columns=['mm'])
-    Sdb['position'] = Sdb['position'].astype(int)
+    Sdb.loc[:, 'position'] = Sdb['position'].astype(int)
 
     # Filter out SNPs that shouldn't be profiled like this
-    Sdb = Sdb[Sdb['cryptic'] == False]
-    Sdb = Sdb.drop(columns="cryptic")
+    # Sdb = Sdb[Sdb['cryptic'] == False]
+    # Sdb = Sdb.drop(columns="cryptic")
 
     if 'morphia' in Sdb.columns:
         col = 'morphia'
@@ -666,9 +667,13 @@ def characterize_SNPs(gdb, Sdb, gene2sequence):
             # Make the new sequence
             snp_start = row['position'] - db['start'].tolist()[0]
             new_sequence = original_sequence.tomutable()
-            new_sequence[snp_start] = row['var_base']
-            if new_sequence[snp_start] == original_sequence[snp_start]:
+            try:
                 new_sequence[snp_start] = row['con_base']
+            except:
+                print(f'ERROR!! {row} {db["gene"].tolist()[0]} {len(new_sequence)}\n{new_sequence}\n{db}')
+
+            if new_sequence[snp_start] == original_sequence[snp_start]:
+                new_sequence[snp_start] = row['var_base']
             new_sequence = new_sequence.toseq()
 
             # Translate
@@ -782,13 +787,21 @@ def parse_prodigal_genes(gene_fasta):
         table['partial'].append('partial=01' in record.description)
 
         # NOTE: PRODIGAL USES A 1-BASED INDEX AND WE USE 0, SO CONVERT TO 0 HERE
-        table['start'].append(int(record.description.split("#")[1].strip())-1)
-        table['end'].append(int(record.description.split("#")[2].strip())-1)
+        start = int(record.description.split("#")[1].strip())-1
+        end = int(record.description.split("#")[2].strip())-1
+        table['start'].append(start)
+        table['end'].append(end)
 
         if scaff not in scaff2gene2sequence:
             scaff2gene2sequence[scaff] = {}
         scaff2gene2sequence[scaff][gene] = record.seq
 
+        # Ensure the start and ends are parsed correctly
+        if (end - start) + 1 != len(record.seq):
+            logging.error(f"ERROR PARSING GENE {gene}; start={start} end={end} length={len(record.seq)}")
+            logging.error(f"Make sure that you report genes with 1-based indexing and with an INCLUSIVE end")
+
+
     scaffs = list(scaff2geneinfo.keys())
     for scaff in scaffs:
         scaff2geneinfo[scaff] = pd.DataFrame(scaff2geneinfo[scaff])
