@@ -13,6 +13,9 @@ import numpy as np
 import pandas as pd
 import Bio.Seq
 
+from functools import lru_cache
+from collections import defaultdict
+
 if __name__ != '__main__':
     from ._version import __version__
     #import inStrain.profileUtilities
@@ -104,7 +107,7 @@ class SNVprofile:
 
         # Add the value if not
         else:
-            Adb = Adb.append(adb)
+            Adb = pd.concat([Adb, adb])
 
         self._store_attributes_file(Adb)
 
@@ -167,6 +170,8 @@ class SNVprofile:
         '''
 
         report_mm_level = kwargs.get('mm_level', False)
+        force_compress = kwargs.get('force_compress', False)
+
         if name == 'SNVs':
             column_order = ['scaffold', 'position', 'position_coverage', 'allele_count',
                             'ref_base', 'con_base', 'var_base',
@@ -175,7 +180,10 @@ class SNVprofile:
                             'gene', 'mutation', 'mutation_type', 'cryptic']
 
             # Get the base table
-            db = self.get_nonredundant_snv_table()
+            try:
+                db = self.get_nonredundant_snv_table()
+            except:
+                db = pd.DataFrame()
 
             # Do you have the mutation types? If so, merge in
             mdb = self.get('SNP_mutation_types')
@@ -184,9 +192,6 @@ class SNVprofile:
                     mdb = mdb[['scaffold', 'position', 'mutation_type', 'mutation', 'gene']]
                     db = pd.merge(db, mdb, how='left', on=['scaffold', 'position'])
 
-            # Do you have information about coverage? If so, merge in
-            pass
-
             db = reorder_columns(db, column_order)
 
         elif name == 'scaffold_info':
@@ -245,7 +250,9 @@ class SNVprofile:
                     del db[c]
 
             db = reorder_columns(db, column_order)
-            db = db[db['coverage'] > 0]
+            if db is not None:
+                if len(db) > 0:
+                    db = db[db['coverage'] > 0]
 
         elif name == 'genome_info':
             column_order = ['genome', 'coverage', 'breadth', 'nucl_diversity',
@@ -266,20 +273,21 @@ class SNVprofile:
             db = self.get('genome_level_info')
             db = reorder_columns(db, column_order)
 
-            # Get rid of some of those dumb columns
-            read_columns = [r for r in db.columns if r.startswith('reads_')]
-            keep_columns = [r for r in read_columns if r in ['reads_unfiltered_reads',
-                            'reads_unfiltered_pairs', 'reads_mean_PID']]
-            for col in set(read_columns) - set(keep_columns):
-                if col in db:
-                    del db[col]
+            if db is not None:
+                # Get rid of some of those dumb columns
+                read_columns = [r for r in db.columns if r.startswith('reads_')]
+                keep_columns = [r for r in read_columns if r in ['reads_unfiltered_reads',
+                                'reads_unfiltered_pairs', 'reads_mean_PID']]
+                for col in set(read_columns) - set(keep_columns):
+                    if col in db:
+                        del db[col]
 
-            if not report_mm_level:
-                if 'mm' in db.columns:
-                    db = db.sort_values('mm').drop_duplicates(
-                            subset=['genome'], keep='last')\
-                            .sort_values('genome')
-                    del db['mm']
+                if not report_mm_level:
+                    if 'mm' in db.columns:
+                        db = db.sort_values('mm').drop_duplicates(
+                                subset=['genome'], keep='last')\
+                                .sort_values('genome')
+                        del db['mm']
 
         elif name == 'mapping_info':
             column_order = ['scaffold', 'pass_pairing_filter', 'filtered_pairs']
@@ -287,9 +295,15 @@ class SNVprofile:
             db = self.get('mapping_info')
             values = inStrain.filter_reads.write_mapping_info(db, None, **kwargs)
 
-            if store:
+            if store & (db is not None):
                 base = self.get_output_base()
-                location = base + name + '.tsv'
+
+                ft = '.tsv'
+                if force_compress:
+                    ft = '.tsv.gz'
+
+                location = base + name + ft
+
                 os.remove(location) if os.path.exists(location) else None
                 f = open(location, 'a')
                 f.write("# {0}\n".format(' '.join(["{0}:{1}".format(k, v) for k, v in values.items()])))
@@ -325,13 +339,79 @@ class SNVprofile:
                                         'name1', 'name2'], keep='last')\
                         .sort_index().drop(columns=['mm'])
 
+        elif name == 'pooled_SNV_info':
+            column_order = ['scaffold', 'position', 'depth', 'A', 'C', 'T', 'G', 'ref_base', 'con_base', 'var_base',
+                            'sample_detections', 'sample_5x_detections',
+                            'DivergentSite_count', 'SNS_count', 'SNV_count', 'con_SNV_count',
+                            'pop_SNV_count']
+            db = self.get('PMdb')
+            db['position'] = db.index
+            db = db.reset_index(drop=True)
+            db = reorder_columns(db, column_order)
+
+        elif name == 'pooled_SNV_data':
+            DSTdb = self.get('DSTdb')
+
+            # Assign keys
+            sa2k = {sample: i for i, sample in enumerate(set(DSTdb.index.get_level_values(0)))}
+            sc2k = {sample: i for i, sample in enumerate(set(DSTdb['scaffold']))}
+
+            # Create new dataframe with keys
+            db = DSTdb.copy()
+            db['sample'] = [sa2k[s] for s in DSTdb.index.get_level_values(0)]
+            db['scaffold'] = db['scaffold'].map(sc2k).astype(int)
+            db['position'] = DSTdb.index.get_level_values(1).astype(int)
+            db = db.reset_index(drop=True)
+
+            # Reorder
+            db = db[['sample', 'scaffold', 'position', 'A', 'C', 'T', 'G']]
+
+            # Make key table
+            table = defaultdict(list)
+            k2sa = {v: k for k, v in sa2k.items()}
+            k2sc = {v: k for k, v in sc2k.items()}
+            for i in sorted(set(k2sa.keys()).union(set(k2sc.keys()))):
+                table['key'].append(i)
+
+                if i in k2sa:
+                    table['sample'].append(k2sa[i])
+                else:
+                    table['sample'].append(np.nan)
+
+                if i in k2sc:
+                    table['scaffold'].append(k2sc[i])
+                else:
+                    table['scaffold'].append(np.nan)
+            Kdb = pd.DataFrame(table)
+
+            # Save key table
+            if store:
+                out_base = self.get_output_base()
+                ft = '.tsv'
+
+                # If this file is going to be huge, compress it
+                if (len(Kdb) > 1e6) | force_compress:
+                    ft = '.tsv.gz'
+
+                Kdb.to_csv(out_base + name + '_keys' + ft, index=False, sep='\t')
+
         else:
             logging.error("Do not know how to store {0}! Crashing now".format(name))
             assert False
 
+        if db is None:
+            logging.info(f"Could not create {name}")
+            return None
+
         if store:
             out_base = self.get_output_base()
-            db.to_csv(out_base + name + '.tsv', index=False, sep='\t')
+            ft = '.tsv'
+
+            # If this file is going to be huge, compress it
+            if (len(db) > 1e6) | force_compress:
+                ft = '.tsv.gz'
+
+            db.to_csv(out_base + name + ft, index=False, sep='\t')
 
         if return_table:
             return db
@@ -387,11 +467,16 @@ class SNVprofile:
                     .drop_duplicates(subset=['scaffold'], keep='last')\
                     .sort_index().drop(columns=['mm'])
 
-    def get_nonredundant_snv_table(self):
+    def get_nonredundant_snv_table(self, cryptic=False):
         '''
         Get a SNP table with just one line per scaffold
         '''
         scdb = self.get('cumulative_snv_table')
+
+        # v1.6; remove cryptic SNVs
+        if ('cryptic' in scdb) & (cryptic == False):
+            scdb = scdb[scdb['cryptic'] == False]
+
         if (scdb is None) or (len(scdb) == 0):
             return pd.DataFrame()
         else:
@@ -486,7 +571,7 @@ class SNVprofile:
             Adb = self._get_attributes_file()
 
             if not same_versions(Adb.loc['version', 'value'], __version__):
-                error_message = "Warning! Your inStrain folder is from version {0}, while the installed version is {1}.\nIf you experience weird behavior, this might be why".format(
+                error_message = "Warning! Your inStrain folder is from version {0}, while the installed version is {1}. If you experience weird behavior, this might be why".format(
                         Adb.loc['version', 'value'], __version__)
 
                 # This is needed because otherwise will make a logger in error!
@@ -1043,9 +1128,15 @@ def reorder_columns(db, column_order):
 
     Any column not in the column_order will be added to the end
     '''
-    columns = set(db.columns)
-    return db[[c for c in column_order if c in columns] \
-                + list(columns - set(column_order))]
+    if db is None:
+        return db
+
+    if len(db) > 0:
+        columns = set(db.columns)
+        return db[[c for c in column_order if c in columns] \
+                    + list(columns - set(column_order))]
+    else:
+        return db
 
 # if __name__ == '__main__':
 #     parser = argparse.ArgumentParser(description= """
