@@ -33,23 +33,24 @@ def printHelp():
     print('')
     print('                ...::: inStrain v' + __version__ + ' :::...''')
     print('')
-    print('  Matt Olm and Alex Crits-Christoph. MIT License. Banfield Lab, UC Berkeley. 2019''')
+    print('  Matt Olm and Alex Crits-Christoph. MIT License. Banfield Lab, UC Berkeley. 2021''')
     print('''\
 
   Choose one of the operations below for more detailed help. See https://instrain.readthedocs.io for documentation.
   Example: inStrain profile -h
 
-  Workflows:
-    profile         -> Create an inStrain profile (microdiversity analysis) from a mapping.
+  Main operations:
+    profile         -> Create an inStrain profile (microdiversity analysis) from a mapping file
     compare         -> Compare multiple inStrain profiles (popANI, coverage_overlap, etc.)
 
-  Single operations:
-    profile_genes   -> Calculate gene-level metrics on an inStrain profile [DEPRECATED; PROVIDE GENES TO profile]
-    genome_wide     -> Calculate genome-level metrics on an inStrain profile [DEPRECATED; PROVIDE .stb FILES TO profile / compare]
+  Auxiliary operations:
+    check_deps      -> Print a list of dependencies, versions, and whether they're working
     quick_profile   -> Quickly calculate coverage and breadth of a mapping using coverM
     filter_reads    -> Commands related to filtering reads from .bam files
     plot            -> Make figures from the results of "profile" or "compare"
     other           -> Other miscellaneous operations
+    profile_genes   -> [DEPRECATED; functionality within PROFILE] Calculate gene-level metrics on an inStrain profile
+    genome_wide     -> [DEPRECATED; functionality within PROFILE / COMPARE] Calculate genome-level metrics
     ''')
 
 def parse_args(args):
@@ -143,7 +144,9 @@ def parse_args(args):
     Iflags.add_argument("-o", "--output", action="store", default='inStrain', \
         help='Output prefix')
     Iflags.add_argument('--use_full_fasta_header', action='store_true', default=False,
-        help='Instead of using the fasta ID (space in header before space), use the full header. Needed for some mapping tools (including bbMap)')
+                        help='Instead of using the fasta ID (space in header before space), use the full header. Needed for some mapping tools (including bbMap)')
+    Iflags.add_argument('--force_compress', action='store_true', default=False,
+                        help='Force compression of all output files')
 
     profile_parser = subparsers.add_parser("profile",formatter_class=SmartFormatter,\
                     parents = [profile_parent, parent_parser, readfilter_parent, readoutput_parent, variant_parent, genes_parent, geneomewide_parent, mm_parent], add_help=False)
@@ -214,6 +217,8 @@ def parse_args(args):
                         help="Dont create plots at the end of the run.")
     Oflags.add_argument('--group_length', default=10000000,
                         help="How many bp to compare simultaneously (higher will use more RAM and run more quickly)", type=int)
+    Oflags.add_argument('--force_compress', action='store_true', default=False,
+                        help='Force compression of all output files')
 
     Cflags = compare_parser.add_argument_group('GENOME CLUSTERING OPTIONS')
     Cflags.add_argument('-ani', "--ani_threshold", help='popANI threshold to cluster genomes at. Must provide .stb file to do so',
@@ -224,6 +229,14 @@ def parse_args(args):
                             to scipy.cluster.hierarchy.linkage)", default='average',
                            choices={'single', 'complete', 'average', 'weighted', 'centroid', 'median', 'ward'})
 
+    Bflags = compare_parser.add_argument_group('SNV POOLING OPTIONS')
+    Bflags.add_argument('-bams', "--bams",
+                        help='Location of .bam files used during inStrain profile commands; needed to pull low-frequency SNVs.' \
+                        ' MUST BE IN SAME ORDER AS THE INPUT FILES',
+                        nargs='*')
+    Bflags.add_argument('--skip_popANI', action='store_true', default=False,\
+                        help="Only run SNV Pooling; skip other compare operations")
+
 
     # Gflags = compare_parser.add_argument_group('GREEDY CLUSTERING OPTIONS [THIS SECTION IS EXPERIMENTAL!]')
     # Gflags.add_argument('--greedy_clustering', action='store_true', default=False,\
@@ -281,7 +294,7 @@ def parse_args(args):
                         nargs='*', default='a')
 
     POflags = plot_parent.add_argument_group('OPTIONAL FIGURE ADJUSTMENTS')
-    POflags.add_argument("-mb", "--minimum_breadth", default=0, type=float,
+    POflags.add_argument("-mb", "--minimum_breadth", default=0.5, type=float,
                     help= "Minimum breadth of coverage for genome to make it into plot (from 0-1).")
     POflags.add_argument("-g", "--genomes", nargs='*',
                     help= "Only plot genomes with the names provided in this argument")
@@ -343,6 +356,9 @@ def parse_args(args):
     Oflags.add_argument('--old_IS', help="Convert an old inStrain version object to the newer version.")
     Oflags.add_argument('--run_statistics', help='Generate runtime reports for an inStrain run.')
 
+    # Dependency checker
+    dep_parser = subparsers.add_parser("check_deps", formatter_class=SmartFormatter)
+
     '''
     ####### PARSE THE ARGUMENTS ######
     '''
