@@ -17,6 +17,8 @@ import inStrain.readComparer
 import inStrain.compare_utils
 import inStrain.genomeUtilities
 import inStrain.controller
+import inStrain.profile
+import inStrain.profile.samtools_ops
 
 class CompareController(object):
     '''
@@ -144,6 +146,23 @@ class CompareController(object):
                 logging.error(f'genome {self.args.genome} is not in the provided .stb file')
                 raise Exception
 
+        # Load the .bam files
+        self.name2bam = gen_name_2_bam(self.inputs, args.bams, self.kwargs.get('processes', 1))
+
+        i = 0
+        if self.name2bam != {}:
+            self.run_pooling = True
+            for name, bam in self.name2bam.items():
+                input = self.inputs[i]
+                i += 1
+
+                if name != os.path.basename(bam):
+                    logging.error(f"WARNING - .bam file {os.path.basename(bam)} is being used for input {input}"\
+                                  f" even though the .bam used in creation of that input is {name}. Change order of "\
+                                  f"--bams input to fix this if it's not correct")
+        else:
+            self.run_pooling = False
+
     def create_scaffoldcomparison_objects(self):
         """
         Figure out which scaffolds need to be compared
@@ -154,12 +173,15 @@ class CompareController(object):
         else:
             scaffolds_to_compare = None
 
-        # Load ScaffoldComparison objects
+        # Load input2scaffolds
         if self.args.database_mode is True:
-            valid_SCs, scaffold2length = make_scaffoldcomparison_objects(self.inputs, scaffolds_to_compare,
-                                                                         input2scaffolds=self.input2scaffolds)
+            i2s = self.input2scaffolds
         else:
-            valid_SCs, scaffold2length = make_scaffoldcomparison_objects(self.inputs, scaffolds_to_compare)
+            i2s = None
+
+        # Create comparison objects
+        valid_SCs, scaffold2length = make_scaffoldcomparison_objects(self.inputs, scaffolds_to_compare,
+                                                                     input2scaffolds=i2s)
 
         self.SC_objects = valid_SCs
         self.scaffold2length = scaffold2length
@@ -177,27 +199,36 @@ class CompareController(object):
         """
         Run each group one at a time, storing the results in the meantime
         """
+        if self.kwargs.get('skip_popANI', False):
+            return
+
         cdbs = [] # Store scaffold comparison information
         mdbs = [] # Store mismatch locations
         pair2mm2covOverlaps = [] # Store coverage overlap locations
         order = [] # Store order of scaffolds
 
         groups = len(self.scaffold_comparison_groups)
+
+        inStrain.logUtils.log_checkpoint("Compare", "multiprocessing", "start")
         for i, SCgroup in enumerate(self.scaffold_comparison_groups):
             logging.info(f'Running group {i+1} of {groups}')
+
             SCgroup.load_cache()
             results = inStrain.compare_utils.run_compare_multiprocessing(SCgroup.cmd_queue, SCgroup.result_queue,
                                                                          self.null_model, num_to_run=len(SCgroup.scaffolds),
                                                                          **self.kwargs)
             for result in results:
                 if result is not None:
+                    # This means you're not doing any pooling
+                    assert len(result) == 4
                     Cdb, Mdb, pair2mm2covOverlap, scaffold = result
                     for item, lis in zip([Cdb, Mdb, pair2mm2covOverlap, scaffold], [cdbs, mdbs, pair2mm2covOverlaps, order]):
                         lis.append(item)
 
             SCgroup.purge_cache()
 
-        # Process results
+        # Process main results
+        inStrain.logUtils.log_checkpoint("Compare", "multiprocessing", "end")
         self.process_results(cdbs, mdbs, pair2mm2covOverlaps, order)
 
     def process_results(self, cdbs, mdbs, pair2mm2covOverlaps, order):
@@ -232,10 +263,42 @@ class CompareController(object):
         self.mismatch_location_db = Mdb
         self.scaff2pair2mm2overlap = scaff2pair2mm2overlap
 
+    # def process_pooling_results(self, DSTdbs, PMdbs, order):
+    #     """
+    #     Merge and store pooling results
+    #     """
+    #     # Create a "scaffold" categorical
+    #     # https://pandas.pydata.org/pandas-docs/stable/user_guide/categorical.html
+    #     from pandas.api.types import CategoricalDtype
+    #     scaff_cat = CategoricalDtype(order)
+    #
+    #     # Add this category to all the dataframes
+    #     for d, s in zip(DSTdbs, order):
+    #         d['scaffold'] = s
+    #         d['scaffold'] = d['scaffold'].astype(scaff_cat)
+    #
+    #     # Add this category to all the dataframes
+    #     for d, s in zip(PMdbs, order):
+    #         d['scaffold'] = s
+    #         d['scaffold'] = d['scaffold'].astype(scaff_cat)
+    #
+    #     self.DSTdb = pd.concat(DSTdbs)
+    #     self.PMdb = pd.concat(PMdbs).astype(
+    #                 {'A':int, 'C':int, 'T':int, 'G':int, 'depth':int,
+    #                  'sample_detections':int, 'DivergentSite_count':int, 'SNS_count':int, 'SNV_count':int,
+    #                  'con_SNV_count':int, 'pop_SNV_count':int, 'sample_5x_detections':int})
+
     def run_auxillary_processing(self):
         """
         Handle .stb files, database mode, and more
         """
+        # Run pooling
+        if self.run_pooling:
+            PM = inStrain.polymorpher.PoolController(self.SC_objects, self.name2bam)
+            PM.main()
+            self.DSTdb = PM.DSTdb
+            self.PMdb = PM.PMdb
+
         # Calculate s2l (scaffold 2 length)
         s2l = {}
         for SC in self.SC_objects:
@@ -243,7 +306,7 @@ class CompareController(object):
         self.s2l = s2l
 
         # Calculate genome-level results
-        if hasattr(self, 'stb'):
+        if hasattr(self, 'stb') and not self.kwargs.get('skip_popANI', False):
             # Calculate bin 2 length
             # Make bin to length
             b2l = {}
@@ -264,15 +327,20 @@ class CompareController(object):
             self.genomelevel_compare = Gdb
 
         # Cluster genomes
-        if hasattr(self, 'genomelevel_compare'):
+        if hasattr(self, 'genomelevel_compare') and not self.kwargs.get('skip_popANI', False):
             self.run_genome_clustering()
 
     def store_results(self):
         # Store the results in the RC
         inStrain.logUtils.log_checkpoint("Compare", "SaveResults", "start")
 
-        self.RCprof.store('comparisonsTable', self.comparison_db, 'pandas', 'Comparisons between the requested IS objects')
-        self.RCprof.store('scaffold2length', self.s2l, 'dictionary', 'Scaffold to length')
+        if hasattr(self, 'comparison_db'):
+            self.RCprof.store('comparisonsTable', self.comparison_db, 'pandas', 'Comparisons between the requested IS objects')
+
+        if hasattr(self, 's2l'):
+            self.RCprof.store('scaffold2length', self.s2l, 'dictionary', 'Scaffold to length')
+
+        force_compress = vars(self.args).get('force_compress', False)
 
         # Store auxillary things
         if hasattr(self, 'bin2length'):
@@ -280,33 +348,54 @@ class CompareController(object):
         if hasattr(self, 'genomelevel_compare'):
             # ... this is jankey, but I guess it's what we do
             out_base = self.RCprof.get_location('output') + os.path.basename(self.RCprof.get('location')) + '_'
-            self.genomelevel_compare.to_csv(out_base + 'genomeWide_compare.tsv', index=False, sep='\t')
+            name = out_base + 'genomeWide_compare.tsv'
+            if force_compress:
+                name += '.gz'
+            self.genomelevel_compare.to_csv(name, index=False, sep='\t')
         if hasattr(self, 'Cdb'):
             # ... this is jankey, but I guess it's what we do
             out_base = self.RCprof.get_location('output') + os.path.basename(self.RCprof.get('location')) + '_'
-            self.Cdb.to_csv(out_base + 'strain_clusters.tsv', index=False, sep='\t')
+            name = out_base + 'strain_clusters.tsv'
+            if force_compress:
+                name += '.gz'
+            self.Cdb.to_csv(name, index=False, sep='\t')
         if hasattr(self, 'stb'):
             self.RCprof.store('scaffold2bin', self.stb, 'dictionary', 'Dictionary of scaffold 2 bin')
 
         # Make the output files
-        self.RCprof.generate('comparisonsTable')
+        if not self.kwargs.get('skip_popANI', False):
+            self.RCprof.generate('comparisonsTable', **vars(self.args))
 
         # Store scaff2pair2mm2SNPs
         if self.args.store_mismatch_locations:
             self.RCprof.store('pairwise_SNP_locations', self.mismatch_location_db, 'pandas',
                          'A dataframe of scaffold, IS pair, mm level, SNP locations')
-            self.RCprof.generate('pairwise_SNP_locations')
+            self.RCprof.generate('pairwise_SNP_locations', **vars(self.args))
 
         # Store scaff2pair2mm2cov
         if self.args.store_coverage_overlap:
             self.RCprof.store('scaff2pair2mm2cov', self.scaff2pair2mm2overlap, 'special',
                          'A dictionary of scaffold -> IS pair -> mm level -> positions with coverage overlap')
 
+        # Store pooling
+        if hasattr(self, 'DSTdb'):
+            self.RCprof.store('DSTdb', self.DSTdb, 'pickle',
+                             'Dense SNV table (from pooling)')
+
+        if hasattr(self, 'PMdb'):
+            self.RCprof.store('PMdb', self.PMdb, 'pickle',
+                             'Informative SNV table (from pooling)')
+
+        if self.run_pooling and hasattr(self, 'PMdb'):
+            self.RCprof.generate('pooled_SNV_info', **vars(self.args))
+            self.RCprof.generate('pooled_SNV_data', **vars(self.args))
+
         inStrain.logUtils.log_checkpoint("Compare", "SaveResults", "end")
 
         # Make plots
-        if hasattr(self, 'genomelevel_compare'):
-            self.make_plots()
+        if not self.kwargs.get('skip_popANI', False):
+            if hasattr(self, 'genomelevel_compare'):
+                self.make_plots()
 
     def make_plots(self):
         '''
@@ -390,7 +479,7 @@ class ScaffoldComparison(object):
         """
         Return True if this scaffold has more than 2 profiles in it that can be compared
         """
-        assert len(self.names) == len(set(self.names)), "You're comparing ISP profiles that have the same names. No no."
+        assert len(self.names) == len(set(self.names)), "InStrain profiles that have the same name cannot be compared. See https://github.com/MrOlm/inStrain/issues/79 and https://github.com/MrOlm/inStrain/issues/97"
         return len(self.profiles) > 1
 
     def compare(self, OSC):
@@ -427,7 +516,7 @@ class ScaffoldCompareGroup(object):
         self.names = names
         self.scaffolds = scaffolds
 
-    def load_cache(self):
+    def load_cache(self, for_pooling=False):
         """
         Load information from ISPs from disk to this object
         """
@@ -441,25 +530,37 @@ class ScaffoldCompareGroup(object):
         # Load data from the profiles
         name2covT = {}
         name2SNPtable = {}
+        name2Rdic = {}
 
         for S, name in zip(sProfiles, names):
-            name2covT[name] = S.get('covT', scaffolds=scaffolds_to_compare)
+            if not for_pooling:
+                name2covT[name] = S.get('covT', scaffolds=scaffolds_to_compare)
             name2SNPtable[name] = S.get('cumulative_snv_table').rename(
                 columns={'conBase': 'con_base', 'refBase': 'ref_base', 'varBase': 'var_base',
                          'baseCoverage': 'position_coverage'})
+            if for_pooling:
+                name2Rdic[name] = S.get("Rdic")
 
-        # Attach this information to ScaffoldComparison objects
-        for SC in self.ScaffoldComparisons:
-            for name in SC.names:
-                SC.SNPtables.append(inStrain.compare_utils.subset_SNP_table(name2SNPtable[name], SC.scaffold))
-                SC.covTs.append(name2covT[name][SC.scaffold])
+        if for_pooling:
+            self.name2SNPtable = name2SNPtable
+            self.name2Rdic = name2Rdic
 
-        # Set up a command and result queue
-        ctx = multiprocessing.get_context('spawn')
-        self.cmd_queue = ctx.Queue()
-        self.result_queue = ctx.Queue()
-        for SC in self.ScaffoldComparisons:
-            self.cmd_queue.put(SC)
+        if not for_pooling:
+            # Attach this information to ScaffoldComparison objects
+            for SC in self.ScaffoldComparisons:
+                for name in SC.names:
+                    SC.SNPtables.append(inStrain.compare_utils.subset_SNP_table(name2SNPtable[name], SC.scaffold))
+                    SC.covTs.append(name2covT[name][SC.scaffold])
+
+                    if for_pooling:
+                        SC.name2Rdic = {n:r[SC.scaffold] for n, r in name2Rdic.items()}
+
+            # Set up a command and result queue
+            ctx = multiprocessing.get_context('spawn')
+            self.cmd_queue = ctx.Queue()
+            self.result_queue = ctx.Queue()
+            for SC in self.ScaffoldComparisons:
+                self.cmd_queue.put(SC)
 
     def purge_cache(self):
         """
@@ -563,4 +664,18 @@ def establish_SC_groups(valid_SCs, Cdb, group_length):
     # Just do it simple; would be better to use the distance matrix, but oh well
     SC_groups = simple_grouping(valid_SCs, group_length)
 
-    return SC_groups
\ No newline at end of file
+    return SC_groups
+
+def gen_name_2_bam(inputs, bams, processes=1):
+    name2bam = {}
+
+    # See if you have a list of bam objects
+    if type(bams) == type([]):
+        if len(bams) == len(inputs):
+            for profile_loc, bam in zip(inputs, bams):
+                ISP = inStrain.SNVprofile.SNVprofile(profile_loc)
+                name = os.path.basename(ISP.get('bam_loc'))
+
+                name2bam[name] = inStrain.profile.samtools_ops.prepare_bam_fie(bam, processes)
+
+    return name2bam
\ No newline at end of file
