@@ -11,12 +11,12 @@ import pysam
 import logging
 import argparse
 import pandas as pd
+pd.set_option('mode.chained_assignment',None)
 import functools
 import logging
 import struct
 import sys
 
-
 from subprocess import call
 from datetime import datetime
 from collections import defaultdict
@@ -25,11 +25,14 @@ from collections import defaultdict
 import inStrain.profile
 import inStrain.profile.samtools_ops
 
+import inStrain.plotting
+import inStrain.plotting.plotting_controller
+
 import inStrain.filter_reads
+import inStrain.utils
 import inStrain.readComparer
 import inStrain.GeneProfile
 import inStrain.genomeUtilities
-import inStrain.plottingUtilities
 import inStrain.quickProfile
 import inStrain.SNVprofile
 import inStrain.logUtils
@@ -70,6 +73,9 @@ class Controller():
         if args.operation == "other":
             self.other_operation(args)
 
+        if args.operation == "check_deps":
+            self.checkdep_operation(args)
+
         self.shutdown(args)
 
     def profile_operation(self, args):
@@ -91,7 +97,8 @@ class Controller():
         inStrain.quickProfile.main(args)
 
     def plot_operation(self, args):
-        inStrain.plottingUtilities.main(args)
+        inStrain.plotting.plotting_controller.PlottingController(args).main()
+        #inStrain.plottingUtilities.main(args)
 
     def other_operation(self, args):
         # Check if you should convert IS profile
@@ -100,14 +107,19 @@ class Controller():
         if args.run_statistics != None:
             inStrain.logUtils.process_logs(args.run_statistics)
 
+    def checkdep_operation(self, args):
+        message = inStrain.utils.gen_dependency_report()
+        print(message)
+
     def shutdown(self, args):
         try:
             logloc = logging.getLoggerClass().root.handlers[0].baseFilename
         except:
             return
-        logging.debug("inStrain complete; shutting down logger and printing run stats (log location = {0})".format(logloc))
+        logging.debug("inStrain complete; shutting down logger and printing run stats if in debug mode (log location = {0})".format(logloc))
         logging.shutdown()
-        inStrain.logUtils.report_run_stats(logloc, most_recent=True, printToo=args.debug, debug=args.debug)
+        if args.debug:
+            inStrain.logUtils.report_run_stats(logloc, most_recent=True, printToo=args.debug, debug=args.debug)
 
 class ProfileController(object):
     '''
@@ -263,11 +275,23 @@ class ProfileController(object):
         # Print status report
         unfiltered_pairs = self.RR['unfiltered_pairs'].iloc[0]
         filterd_pairs = self.RR['filtered_pairs'].iloc[0]
+
+        unfiltered_singletons = self.RR['unfiltered_singletons'].iloc[0]
+        filtered_singletons = self.RR['filtered_singletons'].iloc[0]
+
         pair_length = self.readlength
 
+        # Calculate metrics
+        if unfiltered_singletons > 0:
+            singleton_percent = ((unfiltered_singletons - filtered_singletons) / unfiltered_singletons) * 100
+        else:
+            singleton_percent = 100
+
+        pair_percent = ((unfiltered_pairs - (filterd_pairs - filtered_singletons)) / unfiltered_pairs) * 100
+
         status = ''
-        status += "{0:.1f}% of reads were removed during filtering\n".format(
-                    ((unfiltered_pairs - filterd_pairs) / unfiltered_pairs)*100)
+        status += "{0:.1f}% of pairs and {1:.1f}% of singletons were removed during filtering\n".format(
+                    pair_percent, singleton_percent)
         status += "{0:,} read pairs remain ({1:#.4g} Gbp)".format(
                     filterd_pairs, (filterd_pairs * pair_length)/1e9)
         logging.info(status)
@@ -317,7 +341,7 @@ class ProfileController(object):
         logging.debug("Writing output files now")
 
         for t in ['SNVs', 'scaffold_info', 'SNVs', 'linkage', 'gene_info']:
-            self.ISP.generate(t)
+            self.ISP.generate(t, **self.kwargs)
         self.ISP.generate('mapping_info', **self.kwargs)
 
     def profile_profile_genes(self):
@@ -425,6 +449,11 @@ def setup_logger(loc):
     logging.debug("inStrain version {0} was run \n".format(__version__))
     logging.debug("!"*80 + '\n')
 
+    try:
+        logging.debug(inStrain.utils.gen_dependency_report())
+    except:
+        logging.warning('Failed to log dependencies')
+
 
 def patch_mp_connection_bpo_17560():
     """
