@@ -57,7 +57,7 @@ class Controller():
 
             # Store the output
             IS.generate('genome_info', **vargs)
-            IS.generate('SNVs')
+            IS.generate('SNVs', **vargs)
 
         else:
             self.prepare_genome_wide(IS, vargs)
@@ -165,6 +165,7 @@ def genomeLevel_from_IS(IS, **kwargs):
 
     # Calculate averaing and summing metrics from the scaffold table
     db = IS.get('cumulative_scaffold_table')
+    db['scaffold'] = db['scaffold'].astype(str)
     gdb = _add_stb(db, stb)
 
     # Handle mm level
@@ -345,7 +346,7 @@ def genomeLevel_coverage_info(covT, bin2scaffolds, relevant_genomes, s2l,
 
         gdb = pd.DataFrame(table)
         gdb['iRep'] = iRep
-        gdb['iRep_GC_corrected'] = iRep_accessory['iRep_GC_corrected']
+        gdb.loc[:,'iRep_GC_corrected'] = iRep_accessory['iRep_GC_corrected']
         dbs.append(gdb)
 
     adb = pd.concat(dbs).reset_index(drop=True)
@@ -414,14 +415,14 @@ def makeGenomeWide(thing, db, stb, b2l=None, **kwargs):
     elif thing == 'readComparer':
         return _genome_wide_readComparer(gdb, stb, b2l, **kwargs)
 
-def _add_stb(db, stb):
+def _add_stb(db, stb, verbose=True):
     gdb = db.copy()
 
     if len(gdb) == 0:
         logging.error('Error- no scaffolds detected.')
         return
 
-    gdb['genome'] = gdb['scaffold'].map(stb)
+    gdb.loc[:,'genome'] = gdb['scaffold'].map(stb)
 
     if len(gdb['genome'].dropna().unique()) == 0:
         logging.error('Error- no genomes detected. Example: stb has scaffold {0}, database has scaffold {1}'.format(
@@ -429,8 +430,9 @@ def _add_stb(db, stb):
         return
 
     if len(gdb[gdb['genome'].isna()]) > 0:
-        logging.info("{0:.2f}% of scaffolds have a genome".format((len(gdb[~gdb['genome'].isna()])/\
-                                                      len(gdb))*100))
+        if verbose:
+            logging.info("{0:.2f}% of scaffolds have a genome".format((len(gdb[~gdb['genome'].isna()])/\
+                                                          len(gdb))*100))
     return gdb
 
 def _validate_b2l(gdb, b2l):
@@ -479,7 +481,7 @@ def _genome_wide_si_2(gdb, stb, b2l, **kwargs):
                 table[col].append(sum([x * y for x, y in zip(df[col].fillna(0), df['length'])]) / b2l[genome])
 
             # Weighted average (over detected scaffold length)
-            df['considered_length'] = [x*y for x,y in zip(df['breadth_minCov'], df['length'])]
+            df.loc[:,'considered_length'] = [x*y for x,y in zip(df['breadth_minCov'], df['length'])]
             considered_leng = float(df['considered_length'].sum())
 
             # To maintain backwards compatibility
