@@ -272,10 +272,11 @@ def get_base_counts_mm(pileupcolumn, R2M):
         list with the counts of [A, C, T, G]
     '''
     table = defaultdict(_4zeros)
+    is_set = type(R2M) == type({})
     for pileupread in pileupcolumn.pileups:
         if not pileupread.is_del and not pileupread.is_refskip:
             try:
-                if type(R2M) == type({}):
+                if is_set:
                     table[R2M[pileupread.alignment.query_name]]\
                     [P2C[pileupread.alignment.query_sequence[pileupread.query_position]]] += 1
                 elif pileupread.alignment.query_name in R2M:
@@ -388,10 +389,13 @@ def merge_profile_worker(sprofile_cmd_queue, Sprofile_dict, Sprofiles,
                 if Sprofile.profile_genes:
                     try:
                         Sprofile.run_profile_genes()
-                    except:
+                    except Exception as e:
                         t = time.strftime('%m-%d %H:%M')
                         Sprofile.merge_log += "\n{1} DEBUG FAILURE GeneException {0}".format(str(Sprofile.scaffold), t)
 
+                        error_code = f"\n{str(e)}\n{str(traceback.format_exc())}"
+                        Sprofile.merge_log += error_code
+
             if c == last_c:
                 try:
                     group_log += inStrain.logUtils.get_group_log('MergeProfile', group_name, 'end')
